//
//  RootViewController.m
//  ActionToolbar
//
//  Created by Craig Hockenberry on 4/30/09.
//  Copyright The Iconfactory 2009. All rights reserved.
//

#import "RootViewController.h"
#import "ActionToolbarAppDelegate.h"


@implementation RootViewController

@synthesize simulatedProcessing;

@synthesize tableView;

@synthesize toolbar;
@synthesize firstBarButtonItem;
@synthesize secondBarButtonItem;
@synthesize thirdBarButtonItem;

- (void)viewDidLoad
{
    [super viewDidLoad];

	self.simulatedProcessing = NO;

/*
	Before you can use the action toolbar, you need to make sure two things are set in Interface Builder:
	
	1. Select the toolbar, and then open the Identity Inspector (Cmd-4). Change the class from UIToolbar to IFActionToolbar.
	
	2. Go to the Connections Inspector (Cmd-2) and set the actionToolbarDelegate. In this example, it's set to this view
	controller. The delegate methods are implemented below (under the IFActionToolbarDelegate #pragma.)
*/

/*
	After you've configured the toolbar in Interface Builder, you can attach action items to the bar button items. Each
	item consist of a title, a target and an action. You'll want to localize the titles.
	
	The delegate methods determine how these items are presented at the time the user taps on on the bar button.
*/
	IFActionToolbarActionItem *actionItem = nil;

	NSMutableArray *actionItems = [NSMutableArray arrayWithCapacity:3];
	actionItem = [[[IFActionToolbarActionItem alloc] initWithTitle:@"One" target:self action:@selector(firstAction:)] autorelease];
	[actionItems addObject:actionItem];
	actionItem = [[[IFActionToolbarActionItem alloc] initWithTitle:@"Two" target:self action:@selector(secondAction:)] autorelease];
	[actionItems addObject:actionItem];
	actionItem = [[[IFActionToolbarActionItem alloc] initWithTitle:@"Three" target:self action:@selector(thirdAction:)] autorelease];
	[actionItems addObject:actionItem];
	actionItem = [[[IFActionToolbarActionItem alloc] initWithTitle:@"Cancel" target:nil action:NULL] autorelease];
	[actionItems addObject:actionItem];
	[toolbar attachActionItems:actionItems toItem:thirdBarButtonItem];
/*
	NOTE: The target and action you currently have on bar button item are overriden when you attach the actions. This is
	done so the action menu can be created and displayed.
*/

	[toolbar update];
}

- (void)setView:(UIView *)view
{
	if (view == nil)
	{
		self.tableView = nil;

		self.toolbar = nil;
		self.firstBarButtonItem = nil;
		self.secondBarButtonItem = nil;
		self.thirdBarButtonItem = nil;
	}
	[super setView:view];
}

- (void)dealloc
{
    [super dealloc];
}


#pragma mark Actions

- (void)finishSimulatedProcessing:(id)sender
{
	self.simulatedProcessing = NO;
	
	[toolbar update];
}

- (IBAction)firstButton:(id)sender
{
/*
	This code simulates a refresh button: it shows how you can adjust the toolbar for asynchronous operations
*/
	self.simulatedProcessing = YES;
	[self performSelector:@selector(finishSimulatedProcessing:) withObject:self afterDelay:6.0f];

	[toolbar update];
}

- (IBAction)secondButton:(id)sender
{
	UIAlertView *alertView = [[[UIAlertView alloc] initWithTitle:@"Second Button" message:@"WHOA YOU PUSHED THE BUTTON" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alertView show];

	[tableView deselectRowAtIndexPath:[tableView indexPathForSelectedRow] animated:YES];
	[toolbar update];
}


- (IBAction)firstAction:(id)sender
{
	UIAlertView *alertView = [[[UIAlertView alloc] initWithTitle:@"First Action" message:@"IF YOU WERE AWESOME YOUD SAY UNO" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alertView show];
}

- (IBAction)secondAction:(id)sender
{
	UIAlertView *alertView = [[[UIAlertView alloc] initWithTitle:@"Second Action" message:@"THE FRENCHIES SAY DOO JUST LIKE WHAT A DOGGIE DOES" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alertView show];
}

- (IBAction)thirdAction:(id)sender
{
	UIAlertView *alertView = [[[UIAlertView alloc] initWithTitle:@"Third Action" message:@"DREI AGAIN AS THEY SAY IN DUTCHLAND" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alertView show];
}

#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView
{
    return 1;
}


- (NSInteger)tableView:(UITableView *)aTableView numberOfRowsInSection:(NSInteger)section
{
    return 20;
}


- (UITableViewCell *)tableView:(UITableView *)aTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [aTableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil)
	{
        cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier] autorelease];
    }

	cell.text = [NSString stringWithFormat:@"Row %d", [indexPath row]];
	
    return cell;
}

- (void)tableView:(UITableView *)aTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	[toolbar update];
}

/*
	Wouldn't it be nice if there was a table delegate method like this? If you have the crystal ball, look
	into it...
*/
/*
- (NSIndexPath *)tableView:(UITableView *)tableView willDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
	[toolbar update];
	
	return indexPath;
}
*/

#pragma mark IFActionToolbarDelgate

/*
	These delegate methods control how the toolbar is presented based upon our application's state and environment.
	
	1. A simulatedProcessing property controls the image for the first toolbar button item and the title of the
	action menu.
	2. The table row selection causes the second toolbar button to be enabled. The row of the selection is also
	checked to show/hide the first item in the action menu.
	3. The device is checked and the second item in the action menu is disabled unless the application is running on
	an iPhone.
	
	The delegate also lets you specify which item is the cancel button and which item is the destructive action.
*/

- (BOOL)enableActionToolbarItem:(UIBarButtonItem *)item
{
	if (item == secondBarButtonItem)
	{
		// The second button on the toolbar is only enabled when there is a row selected in the table.
		NSIndexPath *selection = [tableView indexPathForSelectedRow];
		return (selection != nil);
	}
	
	return YES;
}

- (UIImage *)actionToolbarItemImage:(UIBarButtonItem *)item
{
	UIImage *result = nil;

	if (item == firstBarButtonItem)
	{
		// If the simulated processing is occuring, display a circle image instead of a square.
		if (self.simulatedProcessing)
		{
			result = [UIImage imageNamed:@"circle.png"];
		}
		else
		{
			result = [UIImage imageNamed:@"square.png"];
		}
	}
	
	return result;
}

- (NSString *)actionToolbarItemTitle:(UIBarButtonItem *)item
{
	if (item == thirdBarButtonItem)
	{
		// The title of the action menu is changed depending on whether simulated processing is occuring.
		if (self.simulatedProcessing)
		{
			return @"SHUT UP IM THINKING";
		}
		else
		{
			return @"BORN READY";
		}
	}

	return nil;
}

- (BOOL)showActionToolbarActionItem:(IFActionToolbarActionItem *)actionItem
{
	BOOL result = YES;

	if ([actionItem action] == @selector(firstAction:))
	{
		NSIndexPath *selection = [tableView indexPathForSelectedRow];
		if (selection)
		{
			// Don't show the first action when an odd row is selected.
			if (([selection row] % 2) == 1)
			{
				result = NO;
			}
		}
	}
	else if ([actionItem action] == @selector(secondAction:))
	{
		// Don't show the second action if the device is not an iPhone.
		BOOL isPhone = [[[UIDevice currentDevice] model] isEqualToString:@"iPhone"];
		if (! isPhone)
		{
			result = NO;
		}
	}

	return result;
}

- (BOOL)isCancelActionToolbarActionItem:(IFActionToolbarActionItem *)actionItem
{
	BOOL result = NO;

	// By convention, the action item with a nil target is the cancel button.
	if ([actionItem target] == nil)
	{
		result = YES;
	}
	
	return result;
}

- (BOOL)isDestructiveActionToolbarActionItem:(IFActionToolbarActionItem *)actionItem
{
	BOOL result = NO;

	// The third (last) item in the menu will be displayed as a destructive item (red highlight.)
	if ([actionItem action] == @selector(thirdAction:))
	{
		result = YES;
	}

	return result;
}

@end

